@@ -1,10 +1,9 @@
 /* ============================================================
- * File  : adjustcurves.cpp
- * Author: Gilles Caulier <caulier dot gilles at free.fr>
- * Date  : 2004-12-01
+ * Authors: Gilles Caulier
+ * Date   : 2004-12-01
  * Description : image histogram adjust curves.
  *
- * Copyright 2004-2005 by Gilles Caulier
+ * Copyright 2004-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -22,7 +21,6 @@
 // C++ includes.
 
 #include <cmath>
-#include <cstring>
 
 // Qt includes.
 
@@ -43,12 +41,11 @@
 #include <qtimer.h>
 #include <qhbuttongroup.h> 
 #include <qpixmap.h>
-#include <qcheckbox.h>
 
 // KDE includes.
 
+#include <kconfig.h>
 #include <kcursor.h>
-#include <kdebug.h>
 #include <klocale.h>
 #include <knuminput.h>
 #include <kmessagebox.h>
@@ -62,24 +59,31 @@
 #include <kpopupmenu.h>
 #include <kstandarddirs.h>
 
-// Digikam includes.
-
-#include <digikamheaders.h>
-
 // Local includes.
 
 #include "version.h"
 #include "adjustcurves.h"
+#include "adjustcurves.moc"
 
 namespace DigikamAdjustCurvesImagesPlugin
 {
 
-AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint width, uint height)
-                 : ImageTabDialog(parent, i18n("Adjust Color Curves"), "adjustcurves", 
-                                  true, true, false)
+AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, QString title, QFrame* banner)
+                 : Digikam::ImageDlgBase(parent, title, "adjustcurves", true, false, banner)
 {
-    // Create an empty instance of curves to use.
-    m_curves = new Digikam::ImageCurves();
+    m_destinationPreviewData = 0L;
+
+    Digikam::ImageIface iface(0, 0);
+    uchar *data     = iface.getOriginalImage();
+    int w           = iface.originalWidth();
+    int h           = iface.originalHeight();
+    bool sixteenBit = iface.originalSixteenBit();
+    bool hasAlpha   = iface.originalHasAlpha();
+    m_originalImage = Digikam::DImg(w, h, sixteenBit, hasAlpha ,data);
+    delete [] data;
+
+    m_histoSegments = m_originalImage.sixteenBit() ? 65535 : 255;
+    m_curves = new Digikam::ImageCurves(m_originalImage.sixteenBit());
 
     // About data and help button.
 
@@ -88,19 +92,27 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
                                        digikamimageplugins_version,
                                        I18N_NOOP("An image-histogram-curves adjustment plugin for digiKam."),
                                        KAboutData::License_GPL,
-                                       "(c) 2004-2005, Gilles Caulier",
+                                       "(c) 2004-2007, Gilles Caulier",
                                        0,
                                        "http://extragear.kde.org/apps/digikamimageplugins");
 
     about->addAuthor("Gilles Caulier", I18N_NOOP("Author and maintainer"),
-                     "caulier dot gilles at free.fr");
+                     "caulier dot gilles at gmail dot com");
 
     setAboutData(about);
     
     // -------------------------------------------------------------
 
+    m_previewWidget = new Digikam::ImageWidget("adjustcurves Tool Dialog", plainPage(),
+                                               i18n("<p>You can see here the image's "
+                                                    "curve-adjustments preview. You can pick color on image "
+                                                    "to see the color level corresponding on histogram."));
+    setPreviewAreaWidget(m_previewWidget); 
+
+    // -------------------------------------------------------------
+
     QWidget *gboxSettings = new QWidget(plainPage());
-    QGridLayout* grid = new QGridLayout( gboxSettings, 7, 5, marginHint(), spacingHint());
+    QGridLayout* grid = new QGridLayout( gboxSettings, 5, 5, marginHint(), spacingHint());
 
     QLabel *label1 = new QLabel(i18n("Channel:"), gboxSettings);
     label1->setAlignment ( Qt::AlignRight | Qt::AlignVCenter );
@@ -118,12 +130,9 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
                                        "<b>Blue</b>: display the blue image-channel values.<p>"
                                        "<b>Alpha</b>: display the alpha image-channel values. "
                                        "This channel corresponds to the transparency value and "
-                                       "is supported by some image formats, such as PNG or GIF."));
+                                       "is supported by some image formats, such as PNG or TIF."));
 
     m_scaleBG = new QHButtonGroup(gboxSettings);
-    m_scaleBG->setExclusive(true);
-    m_scaleBG->setFrameShape(QFrame::NoFrame);
-    m_scaleBG->setInsideMargin( 0 );
     QWhatsThis::add( m_scaleBG, i18n("<p>Select here the histogram scale.<p>"
                                      "If the image's maximal counts are small, you can use the linear scale.<p>"
                                      "Logarithmic scale can be used when the maximal counts are big; "
@@ -143,44 +152,70 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
     KGlobal::dirs()->addResourceType("histogram-log", KGlobal::dirs()->kde_default("data") + "digikam/data");
     directory = KGlobal::dirs()->findResourceDir("histogram-log", "histogram-log.png");
     logHistoButton->setPixmap( QPixmap( directory + "histogram-log.png" ) );
-    logHistoButton->setToggleButton(true);       
+    logHistoButton->setToggleButton(true);
+    
+    m_scaleBG->setExclusive(true);
+    m_scaleBG->setButton(Digikam::CurvesWidget::LogScaleHistogram);
+    m_scaleBG->setFrameShape(QFrame::NoFrame);
+    m_scaleBG->setInsideMargin( 0 );
 
     QHBoxLayout* l1 = new QHBoxLayout();
-    l1->addWidget(m_scaleBG);
     l1->addStretch(10);
+    l1->addWidget(m_scaleBG);
     
     grid->addMultiCellWidget(label1, 0, 0, 1, 1);
     grid->addMultiCellWidget(m_channelCB, 0, 0, 2, 2);
     grid->addMultiCellLayout(l1, 0, 0, 4, 5);
     
-    QLabel *label5 = new QLabel(i18n("Type:"), gboxSettings);
-    label5->setAlignment ( Qt::AlignRight | Qt::AlignVCenter);
-    m_typeCB = new QComboBox( false, gboxSettings );
-    m_typeCB->insertItem( i18n("Smooth") );
-    m_typeCB->insertItem( i18n("Free") );
-    m_typeCB->setCurrentText( i18n("Smooth") );
-    QWhatsThis::add( m_typeCB, i18n("<p>Select here the curve type to draw for the current channel.<p>"
-                                    "<b>Smooth</b>: this mode constrains the curve type to a smooth line with tension.<p>"
-                                    "<b>Free</b>: with this mode, you can draw your curve free-hand with the mouse."));
+    // -------------------------------------------------------------
 
-    m_labelPos = new QLabel(gboxSettings);
-                                     
-    grid->addMultiCellWidget(label5, 1, 1, 1, 1);
-    grid->addMultiCellWidget(m_typeCB, 1, 1, 2, 2);
-    grid->addMultiCellWidget(m_labelPos, 2, 2, 1, 5);
+    m_histogramWidget = new Digikam::HistogramWidget(256, 140, gboxSettings, false, true, true);
+    QWhatsThis::add( m_histogramWidget, i18n("<p>Here you can see the target preview image histogram drawing of the "
+                                             "selected image channel. This one is re-computed at any curves "
+                                             "settings changes."));
     
+    grid->addMultiCellWidget(m_histogramWidget, 1, 1, 1, 5);
+
+    // -------------------------------------------------------------
+
     m_vGradient = new Digikam::ColorGradientWidget( Digikam::ColorGradientWidget::Vertical, 10, gboxSettings );
     m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
-    grid->addMultiCellWidget(m_vGradient, 3, 3, 0, 0);
+    grid->addMultiCellWidget(m_vGradient, 2, 2, 0, 0);
 
-    m_curvesWidget = new Digikam::CurvesWidget(256, 256, imageData, width, height, m_curves, gboxSettings);
-    QWhatsThis::add( m_curvesWidget, i18n("<p>This is the curve drawing of the selected image "
-                                          "histogram channel"));
-    grid->addMultiCellWidget(m_curvesWidget, 3, 3, 1, 5);
+    m_curvesWidget = new Digikam::CurvesWidget(256, 256, m_originalImage.bits(), m_originalImage.width(),
+                                               m_originalImage.height(), m_originalImage.sixteenBit(),
+                                               m_curves, gboxSettings);
+    QWhatsThis::add( m_curvesWidget, i18n("<p>This is the curve drawing of the selected channel from original image"));
+    grid->addMultiCellWidget(m_curvesWidget, 2, 2, 1, 5);
     
     m_hGradient = new Digikam::ColorGradientWidget( Digikam::ColorGradientWidget::Horizontal, 10, gboxSettings );
     m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
-    grid->addMultiCellWidget(m_hGradient, 4, 4, 1, 5);
+    grid->addMultiCellWidget(m_hGradient, 3, 3, 1, 5);
+    
+    // -------------------------------------------------------------
+    
+    m_curveType = new QHButtonGroup(gboxSettings);
+    m_curveFree = new QPushButton(m_curveType);
+    m_curveType->insert(m_curveFree, FreeDrawing);
+    KGlobal::dirs()->addResourceType("curvefree", KGlobal::dirs()->kde_default("data") +
+                                     "digikam/data");
+    directory = KGlobal::dirs()->findResourceDir("curvefree", "curvefree.png");
+    m_curveFree->setPixmap( QPixmap( directory + "curvefree.png" ) );
+    m_curveFree->setToggleButton(true);
+    QToolTip::add( m_curveFree, i18n( "Curve free mode" ) );
+    QWhatsThis::add( m_curveFree, i18n("<p>With this button, you can draw your curve free-hand with the mouse."));
+    m_curveSmooth = new QPushButton(m_curveType);
+    m_curveType->insert(m_curveSmooth, SmoothDrawing);
+    KGlobal::dirs()->addResourceType("curvemooth", KGlobal::dirs()->kde_default("data") +
+                                     "digikam/data");
+    directory = KGlobal::dirs()->findResourceDir("curvemooth", "curvemooth.png");
+    m_curveSmooth->setPixmap( QPixmap( directory + "curvemooth.png" ) );
+    m_curveSmooth->setToggleButton(true);
+    QToolTip::add( m_curveSmooth, i18n( "Curve smooth mode" ) );
+    QWhatsThis::add( m_curveSmooth, i18n("<p>With this button, you constrains the curve type to a smooth line with tension."));
+    m_curveType->setExclusive(true);
+    m_curveType->setButton(SmoothDrawing);
+    m_curveType->setFrameShape(QFrame::NoFrame);
     
     // -------------------------------------------------------------
     
@@ -217,60 +252,42 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
                                        "smooth curves point on Red, Green, Blue, and Luminosity channels."));
     m_pickerColorButtonGroup->setExclusive(true);
     m_pickerColorButtonGroup->setFrameShape(QFrame::NoFrame);
+    
+    // -------------------------------------------------------------
 
     m_resetButton = new QPushButton(i18n("&Reset"), gboxSettings);
-    QWhatsThis::add( m_resetButton, i18n("<p>Reset curves' values from the current selected channel."));
+    m_resetButton->setPixmap( SmallIcon("reload_page") );
+    QToolTip::add( m_resetButton, i18n( "Reset current channel curves' values." ) );
+    QWhatsThis::add( m_resetButton, i18n("<p>If you press this button, all curves' values "
+                                         "from the current selected channel "
+                                         "will be reset to the default values."));
 
     QHBoxLayout* l3 = new QHBoxLayout();
+    l3->addWidget(m_curveType);
     l3->addWidget(m_pickerColorButtonGroup);
     l3->addWidget(m_resetButton);
     l3->addStretch(10);
     
-    grid->addMultiCellLayout(l3, 5, 5, 1, 5);
+    grid->addMultiCellLayout(l3, 4, 4, 1, 5);
     
     // -------------------------------------------------------------
             
-    m_overExposureIndicatorBox = new QCheckBox(i18n("Over exposure indicator"), gboxSettings);
-    QWhatsThis::add( m_overExposureIndicatorBox, i18n("<p>If you enable this option, over-exposed pixels "
-                                                      "from the target image preview will be over-colored. "
-                                                      "This will not have an effect on the final rendering."));
-    grid->addMultiCellWidget(m_overExposureIndicatorBox, 6, 6, 1, 5);
-    grid->setRowStretch(7, 10);
-    
+
+    grid->setRowStretch(5, 10);
     setUserAreaWidget(gboxSettings);
 
     // -------------------------------------------------------------
-
-    m_previewOriginalWidget = previewOriginalWidget();
-    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>You can see here the original image. You can pick "
-                                                   "a color on the image using the color "
-                                                   "picker tools to select shadow, middle, and highlight "
-                                                   "tones to adjust the curves' points in the Red, "
-                                                   "Green, Blue, and Luminosity Channels."));
-    m_previewTargetWidget   = previewTargetWidget();
-    QWhatsThis::add( m_previewTargetWidget, i18n("<p>You can see here the image's "
-                                                 "curve-adjustments preview."));
-
-    // -------------------------------------------------------------
     
-    // Reset all parameters to the default values.
-    QTimer::singleShot(0, this, SLOT(slotDefault()));
-
-    // -------------------------------------------------------------
-    
-    connect(m_curvesWidget, SIGNAL(signalMouseMoved(int, int)),
-            this, SLOT(slotPositionChanged(int, int)));
-                        
     connect(m_curvesWidget, SIGNAL(signalCurvesChanged()),
-            this, SLOT(slotEffect()));
+            this, SLOT(slotTimer()));
     
-    connect(m_previewOriginalWidget, SIGNAL(spotPositionChanged(  const QColor &, bool, const QPoint & )),
-            this, SLOT(slotSpotColorChanged( const QColor &, bool ))); 
-            
-    connect(m_overExposureIndicatorBox, SIGNAL(toggled (bool)),
-            this, SLOT(slotEffect()));              
+    connect(m_previewWidget, SIGNAL(spotPositionChangedFromOriginal( const Digikam::DColor &, const QPoint & )),
+            this, SLOT(slotSpotColorChanged( const Digikam::DColor & )));
 
-    connect(m_previewTargetWidget, SIGNAL(signalResized()),
+    connect(m_previewWidget, SIGNAL(spotPositionChangedFromTarget( const Digikam::DColor &, const QPoint & )),
+            this, SLOT(slotColorSelectedFromTarget( const Digikam::DColor & )));
+            
+    connect(m_previewWidget, SIGNAL(signalResized()),
             this, SLOT(slotEffect()));                                                            
             
     // -------------------------------------------------------------
@@ -282,7 +299,7 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
     connect(m_scaleBG, SIGNAL(released(int)),
             this, SLOT(slotScaleChanged(int)));
             
-    connect(m_typeCB, SIGNAL(activated(int)),
+    connect(m_curveType, SIGNAL(clicked(int)),
             this, SLOT(slotCurveTypeChanged(int)));
     
     // -------------------------------------------------------------
@@ -290,48 +307,70 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
 
     connect(m_resetButton, SIGNAL(clicked()),
             this, SLOT(slotResetCurrentChannel()));
+
+    connect(m_pickerColorButtonGroup, SIGNAL(released(int)),
+            this, SLOT(slotPickerColorButtonActived()));
 }
 
 AdjustCurveDialog::~AdjustCurveDialog()
 {
+    m_histogramWidget->stopHistogramComputation();
+
+    if (m_destinationPreviewData) 
+       delete [] m_destinationPreviewData;
+       
+    delete m_histogramWidget;
     delete m_curvesWidget;
+    delete m_previewWidget;
     delete m_curves;
 }
 
-void AdjustCurveDialog::slotSpotColorChanged(const QColor &color, bool release)
+void AdjustCurveDialog::slotPickerColorButtonActived()
 {
+    // Save previous rendering mode and toggle to original image.
+    m_currentPreviewMode = m_previewWidget->getRenderingPreviewMode();
+    m_previewWidget->setRenderingPreviewMode(Digikam::ImageGuideWidget::PreviewOriginalImage);
+}
+
+void AdjustCurveDialog::slotSpotColorChanged(const Digikam::DColor &color)
+{
+    Digikam::DColor sc = color;
+
     if ( m_pickBlack->isOn() )
-       {
+    {
        // Black tonal curves point.
        m_curves->setCurvePoint(Digikam::ImageHistogram::ValueChannel, 1, 
-                               QPoint::QPoint(QMAX(QMAX(color.red(), color.green()), color.blue()), 42));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 1, QPoint::QPoint(color.red(), 42));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 1, QPoint::QPoint(color.green(), 42));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 1, QPoint::QPoint(color.blue(), 42));      
-       m_pickBlack->setOn(!release);
-       }
+                               QPoint(QMAX(QMAX(sc.red(), sc.green()), sc.blue()), 42*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 1, QPoint(sc.red(), 42*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 1, QPoint(sc.green(), 42*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 1, QPoint(sc.blue(), 42*m_histoSegments/256));
+       m_pickBlack->setOn(false);
+    }
     else if ( m_pickGray->isOn() )
-       {
+    {
        // Gray tonal curves point.
        m_curves->setCurvePoint(Digikam::ImageHistogram::ValueChannel, 8, 
-                               QPoint::QPoint(QMAX(QMAX(color.red(), color.green()), color.blue()), 128));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 8, QPoint::QPoint(color.red(), 128));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 8, QPoint::QPoint(color.green(), 128));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 8, QPoint::QPoint(color.blue(), 128));      
-       m_pickGray->setOn(!release);
-       }
+                               QPoint(QMAX(QMAX(sc.red(), sc.green()), sc.blue()), 128*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 8, QPoint(sc.red(), 128*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 8, QPoint(sc.green(), 128*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 8, QPoint(sc.blue(), 128*m_histoSegments/256));
+       m_pickGray->setOn(false);
+    }
     else if ( m_pickWhite->isOn() )
-       {
+    {
        // White tonal curves point.
-       m_curves->setCurvePoint(Digikam::ImageHistogram::ValueChannel, 15, 
-                               QPoint::QPoint(QMAX(QMAX(color.red(), color.green()), color.blue()), 213));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 15, QPoint::QPoint(color.red(), 213));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 15, QPoint::QPoint(color.green(), 213));      
-       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 15, QPoint::QPoint(color.blue(), 213));      
-       m_pickWhite->setOn(!release);
-       }
+       m_curves->setCurvePoint(Digikam::ImageHistogram::ValueChannel, 15,
+                               QPoint(QMAX(QMAX(sc.red(), sc.green()), sc.blue()), 213*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::RedChannel, 15, QPoint(sc.red(), 213*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::GreenChannel, 15, QPoint(sc.green(), 213*m_histoSegments/256));
+       m_curves->setCurvePoint(Digikam::ImageHistogram::BlueChannel, 15, QPoint(sc.blue(), 213*m_histoSegments/256));
+       m_pickWhite->setOn(false);
+    }
     else
+    {
        m_curvesWidget->setCurveGuide(color);
+       return;
+    }
 
     // Calculate Red, green, blue curves.
     
@@ -339,61 +378,68 @@ void AdjustCurveDialog::slotSpotColorChanged(const QColor &color, bool release)
        m_curves->curvesCalculateCurve(i);
     
     m_curvesWidget->repaint(false);
+    
+    // restore previous rendering mode.
+    m_previewWidget->setRenderingPreviewMode(m_currentPreviewMode);
        
     slotEffect();  
 }
 
+void AdjustCurveDialog::slotColorSelectedFromTarget( const Digikam::DColor &color )
+{
+    m_histogramWidget->setHistogramGuideByColor(color);
+}
+
 void AdjustCurveDialog::slotResetCurrentChannel()
 {
     m_curves->curvesChannelReset(m_channelCB->currentItem());
 
     m_curvesWidget->reset();
     slotEffect();
-}
-
-void AdjustCurveDialog::slotPositionChanged(int x, int y)
-{
-    if ( x == -1 && y == -1)
-       m_labelPos->clear();
-    else
-       m_labelPos->setText(i18n("Position: x:%1  y:%2").arg(x).arg(y));
+    m_histogramWidget->reset();
 }
 
 void AdjustCurveDialog::slotEffect()
 {
-    Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
-
-    uint* orgData = ifaceDest->getPreviewData();
-    int   w       = ifaceDest->previewWidth();
-    int   h       = ifaceDest->previewHeight();
+    Digikam::ImageIface* iface = m_previewWidget->imageIface();
+    uchar *orgData             = iface->getPreviewImage();
+    int w                      = iface->previewWidth();
+    int h                      = iface->previewHeight();
+    bool sb                    = iface->previewSixteenBit();
 
     // Create the new empty destination image data space.
-    uint* desData = new uint[w*h];
+    m_histogramWidget->stopHistogramComputation();
+
+    if (m_destinationPreviewData) 
+       delete [] m_destinationPreviewData;
+    
+    m_destinationPreviewData = new uchar[w*h*(sb ? 8 : 4)];
 
     // Calculate the LUT to apply on the image.
-    m_curves->curvesLutSetup(Digikam::ImageHistogram::AlphaChannel, m_overExposureIndicatorBox->isChecked());
+    m_curves->curvesLutSetup(Digikam::ImageHistogram::AlphaChannel);
 
     // Apply the lut to the image.
-    m_curves->curvesLutProcess(orgData, desData, w, h);
+    m_curves->curvesLutProcess(orgData, m_destinationPreviewData, w, h);
 
-    ifaceDest->putPreviewData(desData);
-    m_previewTargetWidget->updatePreview();
+    iface->putPreviewImage(m_destinationPreviewData);
+    m_previewWidget->updatePreview();
 
+    // Update histogram.
+    m_histogramWidget->updateData(m_destinationPreviewData, w, h, sb, 0, 0, 0, false);
     delete [] orgData;
-    delete [] desData;
 }
 
-void AdjustCurveDialog::slotOk()
+void AdjustCurveDialog::finalRendering()
 {
     kapp->setOverrideCursor( KCursor::waitCursor() );
-    Digikam::ImageIface ifaceDest(0, 0);
-
-    uint* orgData = ifaceDest.getOriginalData();
-    int   w       = ifaceDest.originalWidth();
-    int   h       = ifaceDest.originalHeight();
-
+    Digikam::ImageIface* iface = m_previewWidget->imageIface();
+    uchar *orgData             = iface->getOriginalImage();
+    int w                      = iface->originalWidth();
+    int h                      = iface->originalHeight();
+    bool sb                    = iface->originalSixteenBit();
+    
     // Create the new empty destination image data space.
-    uint* desData = new uint[w*h];
+    uchar* desData = new uchar[w*h*(sb ? 8 : 4)];
 
     // Calculate the LUT to apply on the image.
     m_curves->curvesLutSetup(Digikam::ImageHistogram::AlphaChannel);
@@ -401,7 +447,7 @@ void AdjustCurveDialog::slotOk()
     // Apply the lut to the image.
     m_curves->curvesLutProcess(orgData, desData, w, h);
 
-    ifaceDest.putOriginalData(i18n("Adjust Curve"), desData);
+    iface->putOriginalImage(i18n("Adjust Curve"), desData);
     kapp->restoreOverrideCursor();
 
     delete [] orgData;
@@ -412,79 +458,151 @@ void AdjustCurveDialog::slotOk()
 void AdjustCurveDialog::slotChannelChanged(int channel)
 {
     switch(channel)
-       {
+    {
        case LuminosityChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::ValueHistogram;
           m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::ValueHistogram;
           m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
           break;
        
-       case RedChannel:
+        case RedChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::RedChannelHistogram;
           m_hGradient->setColors( QColor( "black" ), QColor( "red" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::RedChannelHistogram;
           m_vGradient->setColors( QColor( "red" ), QColor( "black" ) );
           break;
 
        case GreenChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::GreenChannelHistogram;
           m_hGradient->setColors( QColor( "black" ), QColor( "green" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::GreenChannelHistogram;
           m_vGradient->setColors( QColor( "green" ), QColor( "black" ) );
           break;
 
        case BlueChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::BlueChannelHistogram;
           m_hGradient->setColors( QColor( "black" ), QColor( "blue" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::BlueChannelHistogram;
           m_vGradient->setColors( QColor( "blue" ), QColor( "black" ) );
           break;
 
        case AlphaChannel:
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::AlphaChannelHistogram;
           m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::AlphaChannelHistogram;
           m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
           break;
-       }
+    }
 
-    m_typeCB->setCurrentItem(m_curves->getCurveType(channel));  
+    m_curveType->setButton(m_curves->getCurveType(channel));
                 
     m_curvesWidget->repaint(false);
+    m_histogramWidget->repaint(false);
 }
 
 void AdjustCurveDialog::slotScaleChanged(int scale)
 {
     m_curvesWidget->m_scaleType = scale;
+    m_histogramWidget->m_scaleType = scale;
+    m_histogramWidget->repaint(false);
     m_curvesWidget->repaint(false);
 }
 
 void AdjustCurveDialog::slotCurveTypeChanged(int type)
 {
     switch(type)
-       {
+    {
        case SmoothDrawing:          
-          {
+       {
           m_curves->setCurveType(m_curvesWidget->m_channelType, Digikam::ImageCurves::CURVE_SMOOTH);
           m_pickerColorButtonGroup->setEnabled(true);
           break;
-          }
+       }
        
        case FreeDrawing:          
-          {
+       {
           m_curves->setCurveType(m_curvesWidget->m_channelType, Digikam::ImageCurves::CURVE_FREE);
           m_pickerColorButtonGroup->setEnabled(false);
           break;
-          }
        }
+    }
     
     m_curvesWidget->curveTypeChanged();
 }
 
-// Reset all settings.
-void AdjustCurveDialog::slotDefault()
+void AdjustCurveDialog::readUserSettings()
+{
+    KConfig* config = kapp->config();
+    config->setGroup("adjustcurves Tool Dialog");
+
+    m_channelCB->setCurrentItem(config->readNumEntry("Histogram Channel", 0));    // Luminosity.
+    m_scaleBG->setButton(config->readNumEntry("Histogram Scale", Digikam::HistogramWidget::LogScaleHistogram));
+
+    m_curvesWidget->reset();
+
+    for (int i = 0 ; i < 5 ; i++)
+    {
+        m_curves->curvesChannelReset(i);
+        m_curves->setCurveType(i, (Digikam::ImageCurves::CurveType)config->readNumEntry(QString("CurveTypeChannel%1").arg(i),
+                                                                                        Digikam::ImageCurves::CURVE_SMOOTH));
+
+        for (int j = 0 ; j < 17 ; j++)
+        {
+            QPoint disable(-1, -1);
+            QPoint p = config->readPointEntry(QString("CurveAjustmentChannel%1Point%2").arg(i).arg(j), &disable);
+    
+            if (m_originalImage.sixteenBit() && p.x() != -1)
+            {
+                p.setX(p.x()*255);
+                p.setY(p.y()*255);
+            }
+    
+            m_curves->setCurvePoint(i, j, p);
+        }
+ 
+        m_curves->curvesCalculateCurve(i);
+    }
+
+    slotChannelChanged(m_channelCB->currentItem());
+    slotScaleChanged(m_scaleBG->selectedId());
+}
+
+void AdjustCurveDialog::writeUserSettings()
+{
+    KConfig* config = kapp->config();
+    config->setGroup("adjustcurves Tool Dialog");
+    config->writeEntry("Histogram Channel", m_channelCB->currentItem());
+    config->writeEntry("Histogram Scale", m_scaleBG->selectedId());
+
+    for (int i = 0 ; i < 5 ; i++)
+    {
+        config->writeEntry(QString("CurveTypeChannel%1").arg(i), m_curves->getCurveType(i));
+        
+        for (int j = 0 ; j < 17 ; j++)
+        {
+            QPoint p = m_curves->getCurvePoint(i, j);
+    
+            if (m_originalImage.sixteenBit() && p.x() != -1)
+            {
+                p.setX(p.x()/255);
+                p.setY(p.y()/255);
+            }
+    
+            config->writeEntry(QString("CurveAjustmentChannel%1Point%2").arg(i).arg(j), p);
+        }
+    }
+
+    config->sync();
+}
+
+void AdjustCurveDialog::resetValues()
 {
     for (int channel = 0 ; channel < 5 ; channel++)
        m_curves->curvesChannelReset(channel);
 
     m_curvesWidget->reset();
-    slotEffect();
+    m_histogramWidget->reset();
 }
 
 // Load all settings.
@@ -499,10 +617,10 @@ void AdjustCurveDialog::slotUser3()
        return;
 
     if ( m_curves->loadCurvesFromGimpCurvesFile( loadCurvesFile ) == false )
-       {
+    {
        KMessageBox::error(this, i18n("Cannot load from the Gimp curves text file."));
        return;
-       }
+    }
 
     // Refresh the current curves config.
     slotChannelChanged(m_channelCB->currentItem());
@@ -521,10 +639,10 @@ void AdjustCurveDialog::slotUser2()
        return;
 
     if ( m_curves->saveCurvesToGimpCurvesFile( saveCurvesFile ) == false )
-       {
+    {
        KMessageBox::error(this, i18n("Cannot save to the Gimp curves text file."));
        return;
-       }
+    }
 
     // Refresh the current curves config.
     slotChannelChanged(m_channelCB->currentItem());
@@ -532,5 +650,3 @@ void AdjustCurveDialog::slotUser2()
 
 }  // NameSpace DigikamAdjustCurvesImagesPlugin
 
-#include "adjustcurves.moc"
-
