@@ -1,10 +1,9 @@
 /* ============================================================
- * File  : adjustlevels.cpp
- * Author: Gilles Caulier <caulier dot gilles at free.fr>
- * Date  : 2004-07-20
+ * Authors: Gilles Caulier
+ * Date   : 2004-07-20
  * Description : image histogram adjust levels.
  *
- * Copyright 2004-2005 by Gilles Caulier
+ * Copyright 2004-2007 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -22,7 +21,6 @@
 // C++ includes.
 
 #include <cmath>
-#include <cstring>
 
 // Qt includes.
 
@@ -43,12 +41,11 @@
 #include <qtimer.h>
 #include <qhbuttongroup.h> 
 #include <qpixmap.h>
-#include <qcheckbox.h>
 
 // KDE includes.
 
+#include <kconfig.h>
 #include <kcursor.h>
-#include <kdebug.h>
 #include <klocale.h>
 #include <knuminput.h>
 #include <kmessagebox.h>
@@ -62,24 +59,31 @@
 #include <kpopupmenu.h>
 #include <kstandarddirs.h>
 
-// Digikam includes.
-
-#include <digikamheaders.h>
-
 // Local includes.
 
 #include "version.h"
 #include "adjustlevels.h"
+#include "adjustlevels.moc"
 
 namespace DigikamAdjustLevelsImagesPlugin
 {
 
-AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint width, uint height)
-                 : ImageTabDialog(parent, i18n("Adjust Color Levels"), "adjustlevels", 
-                                  true, true, false)
+AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, QString title, QFrame* banner)
+                 : Digikam::ImageDlgBase(parent, title, "adjustlevels", true, false, banner)
 {
-    // Create an empty instance of levels to use.
-    m_levels = new Digikam::ImageLevels();
+    m_destinationPreviewData = 0L;
+
+    Digikam::ImageIface iface(0, 0);
+    uchar *data     = iface.getOriginalImage();
+    int w           = iface.originalWidth();
+    int h           = iface.originalHeight();
+    bool sixteenBit = iface.originalSixteenBit();
+    bool hasAlpha   = iface.originalHasAlpha();
+    m_originalImage = Digikam::DImg(w, h, sixteenBit, hasAlpha ,data);
+    delete [] data;
+
+    m_histoSegments = m_originalImage.sixteenBit() ? 65535 : 255;
+    m_levels = new Digikam::ImageLevels(m_originalImage.sixteenBit());
 
     // About data and help button.
 
@@ -88,15 +92,23 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
                                        digikamimageplugins_version,
                                        I18N_NOOP("An image-histogram-levels adjustment plugin for digiKam."),
                                        KAboutData::License_GPL,
-                                       "(c) 2004-2005, Gilles Caulier",
+                                       "(c) 2004-2007, Gilles Caulier",
                                        0,
                                        "http://extragear.kde.org/apps/digikamimageplugins");
 
     about->addAuthor("Gilles Caulier", I18N_NOOP("Author and maintainer"),
-                     "caulier dot gilles at free.fr");
+                     "caulier dot gilles at gmail dot com");
 
     setAboutData(about);
-    
+
+    // -------------------------------------------------------------
+
+    m_previewWidget = new Digikam::ImageWidget("adjustlevels Tool Dialog", plainPage(),
+                                               i18n("<p>Here you can see the image's "
+                                                    "level-adjustments preview. You can pick color on image "
+                                                    "to see the color level corresponding on histogram."));
+    setPreviewAreaWidget(m_previewWidget); 
+
     // -------------------------------------------------------------
 
     QWidget *gboxSettings = new QWidget(plainPage());
@@ -118,7 +130,7 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
                                        "<b>Blue</b>: display the blue image-channel values.<p>"
                                        "<b>Alpha</b>: display the alpha image-channel values. "
                                        "This channel corresponds to the transparency value and "
-                                       "is supported by some image formats, such as PNG or GIF."));
+                                       "is supported by some image formats, such as PNG or TIF."));
 
     m_scaleBG = new QHButtonGroup(gboxSettings);
     m_scaleBG->setExclusive(true);
@@ -128,7 +140,7 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
                                      "If the image's maximal counts are small, you can use the linear scale.<p>"
                                      "Logarithmic scale can be used when the maximal counts are big; "
                                      "if it is used, all values (small and large) will be visible on the graph."));
-    
+
     QPushButton *linHistoButton = new QPushButton( m_scaleBG );
     QToolTip::add( linHistoButton, i18n( "<p>Linear" ) );
     m_scaleBG->insert(linHistoButton, Digikam::HistogramWidget::LinScaleHistogram);
@@ -136,70 +148,76 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     QString directory = KGlobal::dirs()->findResourceDir("histogram-lin", "histogram-lin.png");
     linHistoButton->setPixmap( QPixmap( directory + "histogram-lin.png" ) );
     linHistoButton->setToggleButton(true);
-    
+
     QPushButton *logHistoButton = new QPushButton( m_scaleBG );
     QToolTip::add( logHistoButton, i18n( "<p>Logarithmic" ) );
     m_scaleBG->insert(logHistoButton, Digikam::HistogramWidget::LogScaleHistogram);
     KGlobal::dirs()->addResourceType("histogram-log", KGlobal::dirs()->kde_default("data") + "digikam/data");
     directory = KGlobal::dirs()->findResourceDir("histogram-log", "histogram-log.png");
     logHistoButton->setPixmap( QPixmap( directory + "histogram-log.png" ) );
-    logHistoButton->setToggleButton(true);       
+    logHistoButton->setToggleButton(true);
 
     QHBoxLayout* l1 = new QHBoxLayout();
     l1->addWidget(label1);
     l1->addWidget(m_channelCB);
-    l1->addWidget(m_scaleBG);
     l1->addStretch(10);
-    
-    grid->addMultiCellLayout(l1, 0, 0, 0, 3);
+    l1->addWidget(m_scaleBG);
+
+    grid->addMultiCellLayout(l1, 0, 0, 0, 4);
 
     // -------------------------------------------------------------
 
-    QFrame *frame = new QFrame(gboxSettings);
-    frame->setFrameStyle(QFrame::NoFrame);
-    QVBoxLayout* l2 = new QVBoxLayout(frame, 2, 0);
-    m_histogramWidget = new Digikam::HistogramWidget(256, 140, imageData, width, height, frame, false);
-    QWhatsThis::add( m_histogramWidget, i18n("<p>This is the histogram drawing of the selected image channel"));
-    l2->addWidget(m_histogramWidget, 0);
-    grid->addMultiCellWidget(frame, 1, 1, 0, 4);
+    m_histogramWidget = new Digikam::HistogramWidget(256, 140, gboxSettings, false, true, true);
+    QWhatsThis::add( m_histogramWidget, i18n("<p>Here you can see the target preview image histogram drawing of the "
+                                             "selected image channel. This one is re-computed at any levels "
+                                             "settings changes."));
+    
+    grid->addMultiCellWidget(m_histogramWidget, 1, 1, 0, 4);
+    
+    // -------------------------------------------------------------
+
+    m_levelsHistogramWidget = new Digikam::HistogramWidget(256, 140, m_originalImage.bits(), m_originalImage.width(),
+                                                     m_originalImage.height(), m_originalImage.sixteenBit(), gboxSettings, false);
+    QWhatsThis::add( m_levelsHistogramWidget, i18n("<p>This is the histogram drawing of the selected channel from original image"));
+    grid->addMultiCellWidget(m_levelsHistogramWidget, 2, 2, 0, 4);
 
     // -------------------------------------------------------------
     
     m_hGradientMinInput = new KGradientSelector( KSelector::Horizontal, gboxSettings );
     m_hGradientMinInput->setFixedHeight( 20 );
     m_hGradientMinInput->setMinValue(0);
-    m_hGradientMinInput->setMaxValue(255);
+    m_hGradientMinInput->setMaxValue(m_histoSegments);
     QWhatsThis::add( m_hGradientMinInput, i18n("<p>Select here the minimal intensity input value of the histogram."));
     QToolTip::add( m_hGradientMinInput, i18n( "Minimal intensity input." ) );
     m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
-    grid->addMultiCellWidget(m_hGradientMinInput, 2, 2, 0, 4);
+    grid->addMultiCellWidget(m_hGradientMinInput, 3, 3, 0, 4);
 
     m_hGradientMaxInput = new KGradientSelector( KSelector::Horizontal, gboxSettings );
     m_hGradientMaxInput->setFixedHeight( 20 );
     m_hGradientMaxInput->setMinValue(0);
-    m_hGradientMaxInput->setMaxValue(255);
+    m_hGradientMaxInput->setMaxValue(m_histoSegments);
     QWhatsThis::add( m_hGradientMaxInput, i18n("<p>Select here the maximal intensity input value of the histogram."));
     QToolTip::add( m_hGradientMaxInput, i18n( "Maximal intensity input." ) );
     m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
-    grid->addMultiCellWidget(m_hGradientMaxInput, 3, 3, 0, 4);
+    grid->addMultiCellWidget(m_hGradientMaxInput, 4, 4, 0, 4);
 
-    m_minInput = new QSpinBox(0, 255, 1, gboxSettings);
+    m_minInput = new QSpinBox(0, m_histoSegments, 1, gboxSettings);
     m_minInput->setValue(0);
     QWhatsThis::add( m_minInput, i18n("<p>Select here the minimal intensity input value of the histogram."));
     QToolTip::add( m_minInput, i18n( "Minimal intensity input." ) );
     m_gammaInput = new KDoubleNumInput(gboxSettings);
     m_gammaInput->setPrecision(2);
-    m_gammaInput->setRange(0.1, 10.0, 0.1);
+    m_gammaInput->setRange(0.1, 3.0, 0.01);
     m_gammaInput->setValue(1.0);
     QToolTip::add( m_gammaInput, i18n( "Gamma input value." ) );
     QWhatsThis::add( m_gammaInput, i18n("<p>Select here the gamma input value."));
-    m_maxInput = new QSpinBox(0, 255, 1, gboxSettings);
-    m_maxInput->setValue(255);
+    m_maxInput = new QSpinBox(0, m_histoSegments, 1, gboxSettings);
+    m_maxInput->setValue(m_histoSegments);
     QToolTip::add( m_maxInput, i18n( "Maximal intensity input." ) );
     QWhatsThis::add( m_maxInput, i18n("<p>Select here the maximal intensity input value of the histogram."));
-    grid->addMultiCellWidget(m_minInput, 2, 2, 5, 5);
-    grid->addMultiCellWidget(m_maxInput, 3, 3, 5, 5);
-    grid->addMultiCellWidget(m_gammaInput, 4, 4, 0, 5);
+    grid->addMultiCellWidget(m_minInput, 3, 3, 5, 5);
+    grid->addMultiCellWidget(m_maxInput, 4, 4, 5, 5);
+    grid->addMultiCellWidget(m_gammaInput, 5, 5, 0, 5);
 
     m_hGradientMinOutput = new KGradientSelector( KSelector::Horizontal, gboxSettings );
     m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
@@ -207,8 +225,8 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     QToolTip::add( m_hGradientMinOutput, i18n( "Minimal intensity output." ) );
     m_hGradientMinOutput->setFixedHeight( 20 );
     m_hGradientMinOutput->setMinValue(0);
-    m_hGradientMinOutput->setMaxValue(255);
-    grid->addMultiCellWidget(m_hGradientMinOutput, 5, 5, 0, 4);
+    m_hGradientMinOutput->setMaxValue(m_histoSegments);
+    grid->addMultiCellWidget(m_hGradientMinOutput, 6, 6, 0, 4);
 
     m_hGradientMaxOutput = new KGradientSelector( KSelector::Horizontal, gboxSettings );
     m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "white" ) );
@@ -216,19 +234,19 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     QToolTip::add( m_hGradientMaxOutput, i18n( "Maximal intensity output." ) );
     m_hGradientMaxOutput->setFixedHeight( 20 );
     m_hGradientMaxOutput->setMinValue(0);
-    m_hGradientMaxOutput->setMaxValue(255);
-    grid->addMultiCellWidget(m_hGradientMaxOutput, 6, 6, 0, 4);
+    m_hGradientMaxOutput->setMaxValue(m_histoSegments);
+    grid->addMultiCellWidget(m_hGradientMaxOutput, 7, 7, 0, 4);
 
-    m_minOutput = new QSpinBox(0, 255, 1, gboxSettings);
+    m_minOutput = new QSpinBox(0, m_histoSegments, 1, gboxSettings);
     m_minOutput->setValue(0);
     QToolTip::add( m_minOutput, i18n( "Minimal intensity output." ) );
     QWhatsThis::add( m_minOutput, i18n("<p>Select here the minimal intensity output value of the histogram."));
-    m_maxOutput = new QSpinBox(0, 255, 1, gboxSettings);
-    m_maxOutput->setValue(255);
+    m_maxOutput = new QSpinBox(0, m_histoSegments, 1, gboxSettings);
+    m_maxOutput->setValue(m_histoSegments);
     QToolTip::add( m_maxOutput, i18n( "Maximal intensity output." ) );
     QWhatsThis::add( m_maxOutput, i18n("<p>Select here the maximal intensity output value of the histogram."));
-    grid->addMultiCellWidget(m_minOutput, 5, 5, 5, 5);
-    grid->addMultiCellWidget(m_maxOutput, 6, 6, 5, 5);
+    grid->addMultiCellWidget(m_minOutput, 6, 6, 5, 5);
+    grid->addMultiCellWidget(m_maxOutput, 7, 7, 5, 5);
 
     // -------------------------------------------------------------
 
@@ -265,11 +283,19 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
                                        "levels input on Red, Green, Blue, and Luminosity channels."));
     m_pickerColorButtonGroup->setExclusive(true);
     m_pickerColorButtonGroup->setFrameShape(QFrame::NoFrame);    
-    
-    m_autoButton = new QPushButton(i18n("&Auto"), gboxSettings);
-    QWhatsThis::add( m_autoButton, i18n("<p>Adjust all levels automatically."));
+
+    m_autoButton = new QPushButton(gboxSettings);
+    m_autoButton->setPixmap( SmallIcon( "run" ) );
+    QToolTip::add( m_autoButton, i18n( "Adjust all levels automatically." ) );
+    QWhatsThis::add( m_autoButton, i18n("<p>If you press this button, all channel levels will be adjusted "
+                                        "automatically."));
+
     m_resetButton = new QPushButton(i18n("&Reset"), gboxSettings);
-    QWhatsThis::add( m_resetButton, i18n("<p>Reset levels values from the current selected channel."));
+    m_resetButton->setPixmap( SmallIcon("reload_page") );
+    QToolTip::add( m_resetButton, i18n( "Reset current channel levels' values." ) );
+    QWhatsThis::add( m_resetButton, i18n("<p>If you press this button, all levels' values "
+                                         "from the current selected channel "
+                                         "will be reset to the default values."));
     
     QHBoxLayout* l3 = new QHBoxLayout();
     l3->addWidget(m_pickerColorButtonGroup);
@@ -277,35 +303,10 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     l3->addWidget(m_resetButton);
     l3->addStretch(10);
     
-    grid->addMultiCellLayout(l3, 7, 7, 0, 4);
-
-    // -------------------------------------------------------------
-    
-    m_overExposureIndicatorBox = new QCheckBox(i18n("Over exposure indicator"), gboxSettings);
-    QWhatsThis::add( m_overExposureIndicatorBox, i18n("<p>If you enable this option, over-exposed pixels "
-                                                      "from the target image preview will be over-colored. "
-                                                      "This will not have an effect on the final rendering."));
-    grid->addMultiCellWidget(m_overExposureIndicatorBox, 8, 8, 0, 5);
+    grid->addMultiCellLayout(l3, 8, 8, 0, 4);
     grid->setRowStretch(9, 10);
     
     setUserAreaWidget(gboxSettings);    
-    
-    // -------------------------------------------------------------
-
-    m_previewOriginalWidget = previewOriginalWidget();
-    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>Here you can see the original image. You can pick "
-                                                   "a color from the image using the color "
-                                                   "picker tools to select shadow, middle, and highlight "
-                                                   "tones to adjust the curves' points in the Red, "
-                                                   "Green, Blue, and Luminosity Channels."));
-    m_previewTargetWidget   = previewTargetWidget();
-    QWhatsThis::add( m_previewTargetWidget, i18n("<p>Here you can see the image's "
-                                                 "level-adjustments preview."));
-    
-    // -------------------------------------------------------------
-    
-    // Reset all parameters to the default values.
-    QTimer::singleShot(0, this, SLOT(slotDefault()));
                     
     // -------------------------------------------------------------
     // Channels and scale selection slots.
@@ -316,19 +317,15 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
     connect(m_scaleBG, SIGNAL(released(int)),
             this, SLOT(slotScaleChanged(int)));
             
-    connect(m_previewOriginalWidget, SIGNAL(spotPositionChanged(  const QColor &, bool, const QPoint & )),
-            this, SLOT(slotSpotColorChanged( const QColor &, bool )));             
+    connect(m_previewWidget, SIGNAL(spotPositionChangedFromOriginal( const Digikam::DColor &, const QPoint & )),
+            this, SLOT(slotSpotColorChanged( const Digikam::DColor & )));
 
-    connect(m_overExposureIndicatorBox, SIGNAL(toggled (bool)),
-            this, SLOT(slotEffect()));      
+    connect(m_previewWidget, SIGNAL(spotPositionChangedFromTarget( const Digikam::DColor &, const QPoint & )),
+            this, SLOT(slotColorSelectedFromTarget( const Digikam::DColor & )));
             
-    connect(m_previewOriginalWidget, SIGNAL(signalResized()),
-            this, SLOT(slotEffect()));
-
-    connect(m_previewTargetWidget, SIGNAL(signalResized()),
+    connect(m_previewWidget, SIGNAL(signalResized()),
             this, SLOT(slotEffect()));                                                            
                         
-                        
     // -------------------------------------------------------------
     // Color sliders and spinbox slots.
 
@@ -367,49 +364,77 @@ AdjustLevelDialog::AdjustLevelDialog(QWidget* parent, uint *imageData, uint widt
 
     connect(m_resetButton, SIGNAL(clicked()),
             this, SLOT(slotResetCurrentChannel()));
+    
+    connect(m_pickerColorButtonGroup, SIGNAL(released(int)),
+            this, SLOT(slotPickerColorButtonActived()));
+            
 }
 
 AdjustLevelDialog::~AdjustLevelDialog()
 {
+    m_histogramWidget->stopHistogramComputation();
+
+    if (m_destinationPreviewData) 
+       delete [] m_destinationPreviewData;
+       
     delete m_histogramWidget;
+    delete m_levelsHistogramWidget;
     delete m_levels;
 }
 
-void AdjustLevelDialog::slotSpotColorChanged(const QColor &color, bool release)
+void AdjustLevelDialog::slotPickerColorButtonActived()
+{
+    // Save previous rendering mode and toggle to original image.
+    m_currentPreviewMode = m_previewWidget->getRenderingPreviewMode();
+    m_previewWidget->setRenderingPreviewMode(Digikam::ImageGuideWidget::PreviewOriginalImage);
+}
+
+void AdjustLevelDialog::slotSpotColorChanged(const Digikam::DColor &color)
 {
     if ( m_pickBlack->isOn() )
-       {
+    {
        // Black tonal levels point.
        m_levels->levelsBlackToneAdjustByColors(m_channelCB->currentItem(), color);      
-       m_pickBlack->setOn(!release);
-       }
+       m_pickBlack->setOn(false);
+    }
     else if ( m_pickGray->isOn() )
-       {
+    {
        // Gray tonal levels point.
        m_levels->levelsGrayToneAdjustByColors(m_channelCB->currentItem(), color);      
-       m_pickGray->setOn(!release);
-       }
+       m_pickGray->setOn(false);
+    }
     else if ( m_pickWhite->isOn() )
-       {
+    {
        // White tonal levels point.
        m_levels->levelsWhiteToneAdjustByColors(m_channelCB->currentItem(), color);      
-       m_pickWhite->setOn(!release);
-       }
+       m_pickWhite->setOn(false);
+    }
     else
-       m_histogramWidget->setHistogramGuide(color);
+    {
+       m_levelsHistogramWidget->setHistogramGuideByColor(color);
+       return;
+    }
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
        
+    // restore previous rendering mode.
+    m_previewWidget->setRenderingPreviewMode(m_currentPreviewMode);
+              
     slotEffect();                
 }
 
+void AdjustLevelDialog::slotColorSelectedFromTarget( const Digikam::DColor &color )
+{
+    m_histogramWidget->setHistogramGuideByColor(color);
+}
+
 void AdjustLevelDialog::slotGammaInputchanged(double val)
 {
     blockSignals(true);
     m_levels->setLevelGammaValue(m_channelCB->currentItem(), val);
     blockSignals(false);
-    slotEffect();
+    slotTimer();
 }
 
 void AdjustLevelDialog::slotAdjustMinInputSpinBox(int val)
@@ -419,11 +444,11 @@ void AdjustLevelDialog::slotAdjustMinInputSpinBox(int val)
     if ( val < m_hGradientMaxInput->value() )
        val = m_hGradientMaxInput->value();
 
-    m_minInput->setValue(255 - val);
+    m_minInput->setValue(m_histoSegments - val);
     m_hGradientMinInput->setValue( val );
-    m_levels->setLevelLowInputValue(m_channelCB->currentItem(), 255 - val);
+    m_levels->setLevelLowInputValue(m_channelCB->currentItem(), m_histoSegments - val);
     blockSignals(false);
-    slotEffect();
+    slotTimer();
 }
 
 void AdjustLevelDialog::slotAdjustMaxInputSpinBox(int val)
@@ -433,11 +458,11 @@ void AdjustLevelDialog::slotAdjustMaxInputSpinBox(int val)
     if ( val > m_hGradientMinInput->value() )
        val = m_hGradientMinInput->value();
 
-    m_maxInput->setValue(255 - val);
+    m_maxInput->setValue(m_histoSegments - val);
     m_hGradientMaxInput->setValue( val );
-    m_levels->setLevelHighInputValue(m_channelCB->currentItem(), 255 - val);
+    m_levels->setLevelHighInputValue(m_channelCB->currentItem(), m_histoSegments - val);
     blockSignals(false);
-    slotEffect();
+    slotTimer();
 }
 
 void AdjustLevelDialog::slotAdjustMinOutputSpinBox(int val)
@@ -447,11 +472,11 @@ void AdjustLevelDialog::slotAdjustMinOutputSpinBox(int val)
     if ( val < m_hGradientMaxOutput->value() )
        val = m_hGradientMaxOutput->value();
 
-    m_minOutput->setValue(255 - val);
+    m_minOutput->setValue(m_histoSegments - val);
     m_hGradientMinOutput->setValue( val );
-    m_levels->setLevelLowOutputValue(m_channelCB->currentItem(), 255 - val);
+    m_levels->setLevelLowOutputValue(m_channelCB->currentItem(), m_histoSegments - val);
     blockSignals(false);
-    slotEffect();
+    slotTimer();
 }
 
 void AdjustLevelDialog::slotAdjustMaxOutputSpinBox(int val)
@@ -461,11 +486,11 @@ void AdjustLevelDialog::slotAdjustMaxOutputSpinBox(int val)
     if ( val > m_hGradientMinOutput->value() )
        val = m_hGradientMinOutput->value();
 
-    m_maxOutput->setValue(255 - val);
+    m_maxOutput->setValue(m_histoSegments - val);
     m_hGradientMaxOutput->setValue( val );
-    m_levels->setLevelHighOutputValue(m_channelCB->currentItem(), 255 - val);
+    m_levels->setLevelHighOutputValue(m_channelCB->currentItem(), m_histoSegments - val);
     blockSignals(false);
-    slotEffect();
+    slotTimer();
 }
 
 void AdjustLevelDialog::slotAdjustSliders()
@@ -477,11 +502,11 @@ void AdjustLevelDialog::slotAdjustSliders()
 
 void AdjustLevelDialog::adjustSliders(int minIn, double gamIn, int maxIn, int minOut, int maxOut)
 {
-    m_hGradientMinInput->setValue(255 - minIn);
-    m_hGradientMaxInput->setValue(255 - maxIn);
+    m_hGradientMinInput->setValue(m_histoSegments - minIn);
+    m_hGradientMaxInput->setValue(m_histoSegments - maxIn);
     m_gammaInput->setValue(gamIn);
-    m_hGradientMinOutput->setValue(255 - minOut);
-    m_hGradientMaxOutput->setValue(255 - maxOut);
+    m_hGradientMinOutput->setValue(m_histoSegments - minOut);
+    m_hGradientMaxOutput->setValue(m_histoSegments - maxOut);
 }
 
 void AdjustLevelDialog::slotResetCurrentChannel()
@@ -490,15 +515,16 @@ void AdjustLevelDialog::slotResetCurrentChannel()
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
-    m_histogramWidget->reset();
+    m_levelsHistogramWidget->reset();
 
     slotEffect();
+    m_histogramWidget->reset();
 }
 
 void AdjustLevelDialog::slotAutoLevels()
 {
     // Calculate Auto levels.
-    m_levels->levelsAuto(m_histogramWidget->m_imageHistogram);
+    m_levels->levelsAuto(m_levelsHistogramWidget->m_imageHistogram);
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
@@ -508,39 +534,46 @@ void AdjustLevelDialog::slotAutoLevels()
 
 void AdjustLevelDialog::slotEffect()
 {
-    Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
-
-    uint* orgData = ifaceDest->getPreviewData();
-    int   w       = ifaceDest->previewWidth();
-    int   h       = ifaceDest->previewHeight();
-
+    Digikam::ImageIface* iface = m_previewWidget->imageIface();
+    uchar *orgData             = iface->getPreviewImage();
+    int w                      = iface->previewWidth();
+    int h                      = iface->previewHeight();
+    bool sb                    = iface->previewSixteenBit();
+    
     // Create the new empty destination image data space.
-    uint* desData = new uint[w*h];
+    m_histogramWidget->stopHistogramComputation();
+
+    if (m_destinationPreviewData) 
+       delete [] m_destinationPreviewData;
+    
+    m_destinationPreviewData = new uchar[w*h*(sb ? 8 : 4)];
 
     // Calculate the LUT to apply on the image.
-    m_levels->levelsLutSetup(Digikam::ImageHistogram::AlphaChannel, m_overExposureIndicatorBox->isChecked());
+    m_levels->levelsLutSetup(Digikam::ImageHistogram::AlphaChannel);
 
     // Apply the lut to the image.
-    m_levels->levelsLutProcess(orgData, desData, w, h);
+    m_levels->levelsLutProcess(orgData, m_destinationPreviewData, w, h);
 
-    ifaceDest->putPreviewData(desData);
-    m_previewTargetWidget->updatePreview();
+    iface->putPreviewImage(m_destinationPreviewData);
+    m_previewWidget->updatePreview();
 
+    // Update histogram.
+    m_histogramWidget->updateData(m_destinationPreviewData, w, h, sb, 0, 0, 0, false);
+    
     delete [] orgData;
-    delete [] desData;
 }
 
-void AdjustLevelDialog::slotOk()
+void AdjustLevelDialog::finalRendering()
 {
     kapp->setOverrideCursor( KCursor::waitCursor() );
-    Digikam::ImageIface ifaceDest(0, 0);
-
-    uint* orgData = ifaceDest.getOriginalData();
-    int   w       = ifaceDest.originalWidth();
-    int   h       = ifaceDest.originalHeight();
+    Digikam::ImageIface* iface = m_previewWidget->imageIface();
+    uchar *orgData             = iface->getOriginalImage();
+    int w                      = iface->originalWidth();
+    int h                      = iface->originalHeight();
+    bool sb                    = iface->originalSixteenBit();
 
     // Create the new empty destination image data space.
-    uint* desData = new uint[w*h];
+    uchar* desData = new uchar[w*h*(sb ? 8 : 4)];
 
     // Calculate the LUT to apply on the image.
     m_levels->levelsLutSetup(Digikam::ImageHistogram::AlphaChannel);
@@ -548,7 +581,7 @@ void AdjustLevelDialog::slotOk()
     // Apply the lut to the image.
     m_levels->levelsLutProcess(orgData, desData, w, h);
 
-    ifaceDest.putOriginalData(i18n("Adjust Level"), desData);
+    iface->putOriginalImage(i18n("Adjust Level"), desData);
     kapp->restoreOverrideCursor();
 
     delete [] orgData;
@@ -559,9 +592,10 @@ void AdjustLevelDialog::slotOk()
 void AdjustLevelDialog::slotChannelChanged(int channel)
 {
     switch(channel)
-       {
+    {
        case LuminosityChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::ValueHistogram;
+          m_levelsHistogramWidget->m_channelType = Digikam::HistogramWidget::ValueHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
@@ -569,7 +603,8 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           break;
        
        case RedChannel:
-          m_histogramWidget->m_channelType = Digikam::HistogramWidget::RedChannelHistogram;
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::RedChannelHistogram;       
+          m_levelsHistogramWidget->m_channelType = Digikam::HistogramWidget::RedChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "red" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "red" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "red" ) );
@@ -577,7 +612,8 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           break;
 
        case GreenChannel:
-          m_histogramWidget->m_channelType = Digikam::HistogramWidget::GreenChannelHistogram;
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::GreenChannelHistogram;       
+          m_levelsHistogramWidget->m_channelType = Digikam::HistogramWidget::GreenChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "green" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "green" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "green" ) );
@@ -586,6 +622,7 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
 
        case BlueChannel:
           m_histogramWidget->m_channelType = Digikam::HistogramWidget::BlueChannelHistogram;
+          m_levelsHistogramWidget->m_channelType = Digikam::HistogramWidget::BlueChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "blue" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "blue" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "blue" ) );
@@ -593,13 +630,14 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
           break;
 
        case AlphaChannel:
-          m_histogramWidget->m_channelType = Digikam::HistogramWidget::AlphaChannelHistogram;
+          m_histogramWidget->m_channelType = Digikam::HistogramWidget::AlphaChannelHistogram;       
+          m_levelsHistogramWidget->m_channelType = Digikam::HistogramWidget::AlphaChannelHistogram;
           m_hGradientMinInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMaxInput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMinOutput->setColors( QColor( "black" ), QColor( "white" ) );
           m_hGradientMaxOutput->setColors( QColor( "black" ), QColor( "white" ) );
           break;
-       }
+    }
 
     adjustSliders(m_levels->getLevelLowInputValue(channel),
                   m_levels->getLevelGammaValue(channel),
@@ -607,26 +645,91 @@ void AdjustLevelDialog::slotChannelChanged(int channel)
                   m_levels->getLevelLowOutputValue(channel),
                   m_levels->getLevelHighOutputValue(channel));
 
-    m_histogramWidget->repaint(false);
+    m_levelsHistogramWidget->repaint(false);
+    m_histogramWidget->repaint(false);    
 }
 
 void AdjustLevelDialog::slotScaleChanged(int scale)
 {
-    m_histogramWidget->m_scaleType = scale;
+    m_levelsHistogramWidget->m_scaleType = scale;
+    m_histogramWidget->m_scaleType       = scale;
     m_histogramWidget->repaint(false);
+    m_levelsHistogramWidget->repaint(false);
 }
 
-// Reset all settings.
-void AdjustLevelDialog::slotDefault()
+void AdjustLevelDialog::readUserSettings()
+{
+    KConfig* config = kapp->config();
+    config->setGroup("adjustlevels Tool Dialog");
+
+    m_channelCB->setCurrentItem(config->readNumEntry("Histogram Channel", 0));    // Luminosity.
+    m_scaleBG->setButton(config->readNumEntry("Histogram Scale", Digikam::HistogramWidget::LogScaleHistogram));
+
+    for (int i = 0 ; i < 5 ; i++)
+    {
+        bool sb        = m_originalImage.sixteenBit();
+        int max        = sb ? 65535 : 255;
+        double gamma   = config->readDoubleNumEntry(QString("GammaChannel%1").arg(i), 1.0);
+        int lowInput   = config->readNumEntry(QString("LowInputChannel%1").arg(i), 0);
+        int lowOutput  = config->readNumEntry(QString("LowOutputChannel%1").arg(i), 0);
+        int highInput  = config->readNumEntry(QString("HighInputChannel%1").arg(i), max);
+        int highOutput = config->readNumEntry(QString("HighOutputChannel%1").arg(i), max);
+    
+        m_levels->setLevelGammaValue(i, gamma);
+        m_levels->setLevelLowInputValue(i, sb ? lowInput*255 : lowInput);
+        m_levels->setLevelHighInputValue(i, sb ? highInput*255 : highInput);
+        m_levels->setLevelLowOutputValue(i, sb ? lowOutput*255 : lowOutput);
+        m_levels->setLevelHighOutputValue(i, sb ? highOutput*255 : highOutput);
+    }
+
+    m_levelsHistogramWidget->reset();
+    m_histogramWidget->reset();
+    slotChannelChanged(m_channelCB->currentItem());
+    slotScaleChanged(m_scaleBG->selectedId());
+
+    // This is mandatory here to set spinbox values because slot connections 
+    // can be not set completely at plugin startup.
+    m_minInput->setValue(m_levels->getLevelLowInputValue(m_channelCB->currentItem()));
+    m_minOutput->setValue(m_levels->getLevelLowOutputValue(m_channelCB->currentItem()));
+    m_maxInput->setValue(m_levels->getLevelHighInputValue(m_channelCB->currentItem()));
+    m_maxOutput->setValue(m_levels->getLevelHighOutputValue(m_channelCB->currentItem()));
+}
+
+void AdjustLevelDialog::writeUserSettings()
+{
+    KConfig* config = kapp->config();
+    config->setGroup("adjustlevels Tool Dialog");
+    config->writeEntry("Histogram Channel", m_channelCB->currentItem());
+    config->writeEntry("Histogram Scale", m_scaleBG->selectedId());
+
+    for (int i = 0 ; i < 5 ; i++)
+    {
+        bool sb        = m_originalImage.sixteenBit();
+        double gamma   = m_levels->getLevelGammaValue(i);
+        int lowInput   = m_levels->getLevelLowInputValue(i);
+        int lowOutput  = m_levels->getLevelLowOutputValue(i);
+        int highInput  = m_levels->getLevelHighInputValue(i);
+        int highOutput = m_levels->getLevelHighOutputValue(i);
+
+        config->writeEntry(QString("GammaChannel%1").arg(i), gamma);
+        config->writeEntry(QString("LowInputChannel%1").arg(i), sb ? lowInput/255 : lowInput);
+        config->writeEntry(QString("LowOutputChannel%1").arg(i), sb ? lowOutput/255 : lowOutput);
+        config->writeEntry(QString("HighInputChannel%1").arg(i), sb ? highInput/255 : highInput);
+        config->writeEntry(QString("HighOutputChannel%1").arg(i), sb ? highOutput/255 : highOutput);
+    }
+
+    config->sync();
+}
+
+void AdjustLevelDialog::resetValues()
 {
     for (int channel = 0 ; channel < 5 ; ++channel)
        m_levels->levelsChannelReset(channel);
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
+    m_levelsHistogramWidget->reset();
     m_histogramWidget->reset();
-    
-    slotEffect();
 }
 
 // Load all settings.
@@ -641,10 +744,10 @@ void AdjustLevelDialog::slotUser3()
        return;
 
     if ( m_levels->loadLevelsFromGimpLevelsFile( loadLevelsFile ) == false )
-       {
+    {
        KMessageBox::error(this, i18n("Cannot load from the Gimp levels text file."));
        return;
-       }
+    }
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
@@ -662,16 +765,13 @@ void AdjustLevelDialog::slotUser2()
        return;
 
     if ( m_levels->saveLevelsToGimpLevelsFile( saveLevelsFile ) == false )
-       {
+    {
        KMessageBox::error(this, i18n("Cannot save to the Gimp levels text file."));
        return;
-       }
+    }
 
     // Refresh the current levels config.
     slotChannelChanged(m_channelCB->currentItem());
 }
 
 }  // NameSpace DigikamAdjustLevelsImagesPlugin
-
-#include "adjustlevels.moc"
-
